using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Principal;
using System.Threading;
using System.Web;
using System.Web.Caching;
using System.Xml.XPath;

namespace JamesKovacs.Web.HttpModules {
    /// <summary>
    /// ImpostorHttpModule allows alternate role memberships to be specified for some users
    /// in a file ~/App_Data/Impostors.xml
    /// </summary>
    public class ImpostorHttpModule : IHttpModule {
        #region Constructors
        public ImpostorHttpModule() {
        }
        #endregion

        #region IHttpModule Members
        public void Dispose() {
            // NOP
        }

        /// <summary>
        /// Initializes a module and prepares it to handle requests.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpApplication"></see> that provides access to the methods, properties, and events common to all application objects within an ASP.NET application</param>
        public void Init(HttpApplication context) {
            context.PostAuthenticateRequest += new EventHandler(context_PostAuthenticateRequest);
        }

        /// <summary>
        /// Handles the PostAuthenticateRequest event of the context control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        private void context_PostAuthenticateRequest(object sender, EventArgs e) {
            string impostorFile = HttpContext.Current.Server.MapPath("~/App_Data/Impostors.xml");
            if(File.Exists(impostorFile)) {
                Dictionary<string, string[]> impostors = HttpContext.Current.Cache[IMPOSTOR_CACHE] as Dictionary<string, string[]>;
                if(impostors == null) {
                    impostors = new Dictionary<string, string[]>();
                    XPathNavigator doc = (new XPathDocument(impostorFile)).CreateNavigator();
                    foreach(XPathNavigator node in doc.Select("/impostors/impostor")) {
                        string name = node.GetAttribute("name", "");
                        string[] roles = node.GetAttribute("roles", "").Split(',');
                        for(int i = 0; i < roles.Length; i++) {
                            roles[i] = roles[i].Trim();
                        }
                        impostors.Add(name.ToLowerInvariant(), roles);
                    }
                    HttpContext.Current.Cache.Insert(IMPOSTOR_CACHE, impostors, new CacheDependency(impostorFile));
                }

                IIdentity identity = HttpContext.Current.User.Identity;
                string key = identity.Name.ToLowerInvariant();
                if(identity.IsAuthenticated && impostors.ContainsKey(key)) {
                    string[] roles = impostors[key];
                    IPrincipal userWithRoles = new GenericPrincipal(identity, roles);
                    HttpContext.Current.User = userWithRoles;
                    Thread.CurrentPrincipal = userWithRoles;
                }
            }
        }
        #endregion

        #region Private data members
        private const string IMPOSTOR_CACHE = "ImpostorCache";
        #endregion
    }
}